'''
Transfers filetype definitions from a MySQL database
to a Windows XP machine's registry.

Algorithm
    Get records
    For each record:
        Get class
        Get Shellex
        Delete
            Delete ext
            Delete class
            Delete _max class
        Assemble New Types
            Create ext
            Create class
        Apply Changes
            Apply ext
            Apply class
    
'''


#High level registry access
from regbatch import *
#Low level registry access
import _winreg
#Database access
import MySQLdb

HKCR = "HKEY_CLASSES_ROOT"
DEFAULT_EXT = {'ext': '""',
               'name': '""',
               'type': '"generic"',
               'mimetype': None,
               'persistenthandler': None,
               'icon': None,
               'clsid': None,
               'defaultAction': '"open"',
               'openCmd': None,
               'editCommand': None,
               'editCmd': 'Open',
               'fixed': '0'}
conn = MySQLdb.connect('localhost', 'root', '', 'filetypes')

FIELD_EXT         = 0
FIELD_NAME        = 1
FIELD_TYPE        = 2
FIELD_MIME        = 3
FIELD_PERSHANDLER = 4
FIELD_ICON        = 5
FIELD_CLSID       = 6
FIELD_ACTIONNAME  = 7
FIELD_DEFAULT     = 8
FIELD_OPENCMD     = 9
FIELD_EDITCMD     = 10


def getExts():
    cursor = conn.cursor()
    
    sql = r'''
          SELECT exts.ext,
                 exts.name,
                 exts.type,
                 exts.mimetype,
                 exts.persistenthandler,
                 exts.icon,
                 exts.clsid,
                 exts.actionname,
                 exts.defaultaction,
                 openHandler.command AS openCmd,
                 editHandler.command AS editCmd
          FROM   exts
                 LEFT JOIN handlers AS openHandler ON exts.openCommand=openHandler.name
                 LEFT JOIN handlers AS editHandler ON exts.editCommand=editHandler.name
          WHERE  fixed = 1
          '''
    
    cursor.execute(sql)
    temp = cursor.fetchall()
    cursor.close()

    return temp


def getRelatedKeys(ext):
    #Get extension key
    try:
        extKey = RegReadKey("%s\\.%s" % (HKCR, ext))
    except RegError:
        extKey = None
        #print "%s: Not found." % (ext)

    #Get class key
    try:
        classKey = RegReadKey("%s\\%s" % (HKCR, extKey.values['']))
    except RegError:
        classKey = None
        #print "%s: Class not found." % (ext)
    except AttributeError:
        classKey = None
        #print "%s: Unclassed." % (ext)
    except KeyError:
        classKey = None

    return (extKey, classKey)


def deleteFiletype(ext, cls, unlock = False):
    #Delete ext
    try:
        if unlock: RegUnlockKey('%s\\.%s' % (HKCR, ext))
        RegDeleteKey('%s\\.%s' % (HKCR, ext))
    except RegError:
        pass
    
    #Delete _max class
    try:
        if unlock: RegUnlockKey('%s\\_max.%s' % (HKCR, ext))
        RegDeleteKey('%s\\_max.%s' % (HKCR, ext))
    except RegError:
        pass

    #Delete class    
    try:
        if cls:
            if unlock: RegUnlockKey('%s\\%s' % (HKCR, cls))
            RegDeleteKey('%s\\%s' % (HKCR, cls))
    except RegError:
        pass


def createExtKey(record):
    extKey = RegKey('.' + record[FIELD_EXT])
    extKey.values[''] = '_max.' + record[FIELD_EXT]
    extKey.values['PerceivedType'] = record[FIELD_TYPE]
    if record[FIELD_MIME]:
        extKey.values['Content Type'] = record[FIELD_MIME]
    if record[FIELD_PERSHANDLER]:
        persKey = RegKey('PersistentHandler')
        persKey.values[''] = record[FIELD_PERSHANDLER]
        extKey.subkeys.append(persKey)
    
    return extKey


def createClassKey(record, extras=None):
    classKey = RegKey('_max.' + record[FIELD_EXT])
    classKey.values[''] = record[FIELD_NAME]
    
    if record[FIELD_ICON]:
        iconKey = RegKey('DefaultIcon')
        iconKey.values[''] = record[FIELD_ICON]
        classKey.subkeys.append(iconKey)
        
    if record[FIELD_CLSID]:
        clsidKey = RegKey('CLSID')
        clsidKey.values[''] = record[FIELD_CLSID]
        classKey.subkeys.append(clsidKey)
        

    if record[FIELD_OPENCMD] or record[FIELD_EDITCMD]:
        shellKey = RegKey('shell')
        shellKey.values[''] = record[FIELD_DEFAULT]
        if record[FIELD_OPENCMD]:
            openKey = RegKey('open')
            openKey.values[''] = record[FIELD_ACTIONNAME]
            openCmdKey = RegKey('command')
            openCmdKey.values[''] = record[FIELD_OPENCMD]
            openKey.subkeys.append(openCmdKey)
            shellKey.subkeys.append(openKey)
        if record[FIELD_EDITCMD]:
            editKey = RegKey('edit')
            editKey.values[''] = 'Edit'
            editCmdKey = RegKey('command')
            editCmdKey.values[''] = record[FIELD_EDITCMD]
            editKey.subkeys.append(editCmdKey)
            shellKey.subkeys.append(editKey)
        classKey.subkeys.append(shellKey)

    if extras:
        if isinstance(extras, RegKey):
            classKey.subkeys.append(extras)
        else:
            for extra in extras:
                classKey.subkeys.append(extra)
    
    return classKey


def applyFiletype(extKey, classKey):
    RegWriteKey(HKCR, extKey, False)
    RegWriteKey(HKCR, classKey, False)


def lockFiletype(extKey, classKey):
    RegLockKey(HKCR + '\\' + extKey.name)
    RegLockKey(HKCR + '\\' + classKey.name)


def processFiletype(record, lock, unlock):
    print record[FIELD_EXT]
    (oldExt, oldClass) = getRelatedKeys(record[FIELD_EXT])

    ext = record[FIELD_EXT]
    extra = None
    if oldExt:
        print oldExt.values
    
        cls = None
        if oldClass:
            cls = oldClass.name
            if oldClass.findSubkey('shellex'):
                extra = oldClass.getSubkey('shellex')
                print '---', extra

        try:
            deleteFiletype(ext, cls, unlock)
        except Exception, e:
            print e
            
    
    extKey   = createExtKey(record)
    classKey = createClassKey(record, extra)

    applyFiletype(extKey, classKey)
    if lock: applyFiletype(extKey, classKey)
    

def main(lock = False, unlock = False):
    records = getExts()

    for record in records:
        processFiletype(record, lock, unlock)

    