'''
Transfers filetype definitions from a Windows
XP machine's registry to a MySQL database.
'''

# NOTE: An overriding key:
# HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts

#High level registry access
from regbatch import *
#Low level registry access
import _winreg
#Database access
import MySQLdb

HKCR = "HKEY_CLASSES_ROOT"
DEFAULT_EXT = {'ext': '',
               'name': '_NEW_TYPE',
               'type': 'generic',
               'mimetype': 'application/octet-stream',
               'persistenthandler': None,
               'icon': None,
               'clsid': None,
               'defaultAction': 'open',
               'openCommand': None,
               'editCommand': None,
               'actionName': 'Open',
               'fixed': '0'}
conn = MySQLdb.connect('localhost', 'root', '', 'filetypes')

def getRelatedKeys(ext):
    #Get extension key
    try:
        extKey = RegReadKey("%s\\.%s" % (HKCR, ext))
    except RegError:
        extKey = None
        #print "%s: Not found." % (ext)

    #Get class key
    try:
        classKey = RegReadKey("%s\\%s" % (HKCR, extKey.values['']))
    except RegError:
        classKey = None
        #print "%s: Class not found." % (ext)
    except AttributeError:
        classKey = None
        #print "%s: Unclassed." % (ext)
    except KeyError:
        classKey = None

    return (extKey, classKey)

def getExts():
    exts = []
    
    i = 1 #Start from the second key, ignoring "*"
    while True:
        ext = _winreg.EnumKey(_winreg.HKEY_CLASSES_ROOT, i)
        if ext[0] != ".": #Stop on first non-ext
            break
        
        hkey = _winreg.OpenKey(_winreg.HKEY_CLASSES_ROOT, ext)
        try:
            curType = _winreg.QueryValueEx(hkey, "PerceivedType")[0]
        except:
            pass
        exts.append(ext[1:])
        hkey.Close()
        i += 1

    return exts

def insertExtRecord(extDict, replace = False, sanitize = True):
    cursor = conn.cursor()

    if extDict['openCommand']:
        extDict['name'] += ' [OPEN: %s' % extDict['openCommand']
        extDict['openCommand'] = None
    if extDict['editCommand']:
        extDict['name'] += ' [EDIT: %s' % extDict['editCommand']
        extDict['editCommand'] = None
        
    extDict['name'] = '_' + extDict['name']

    if sanitize:
        extDictSanitized = {}
        for field in extDict:
            if extDict[field] is None:
                extDictSanitized[field] = 'NULL'
            else:
                extDictSanitized[field] = '"' + MySQLdb.escape_string(extDict[field]) + '"'
    else:
        extDictSanitized = extDict
        
    sql = (replace and 'REPLACE' or 'INSERT') + r'''
          INTO   exts (ext,
                       name,
                       type,
                       mimetype,
                       persistenthandler,
                       icon,
                       clsid,
                       defaultAction,
                       openCommand,
                       editCommand,
                       actionName,
                       fixed)
          VALUES      (%(ext)s,
                       %(name)s,
                       %(type)s,
                       %(mimetype)s,
                       %(persistenthandler)s,
                       %(icon)s,
                       %(clsid)s,
                       %(defaultAction)s,
                       %(openCommand)s,
                       %(editCommand)s,
                       %(actionName)s,
                       %(fixed)s)
          ''' % extDictSanitized

    import re

    try:
        cursor.execute(sql)
    except Exception, e:
        #print e
        print 'ERROR: ' + extDict['ext']
    else:
        #print re.sub(r'\s+', ' ', sql)
        #print 'SUCCESS: ' + extDict['ext']
        print extDict['ext']
    
    cursor.close()

def updateExtRecord(ext, fieldDict, sanitize = True):
    cursor = conn.cursor()

    if sanitize:
        for field in extDict:
                extDict[field] = '"' + MySQLdb.escape_string(extDict[field]) + '"'
    
    sql = r'''
          UPDATE exts
          SET    %s
          WHERE  ext = '%s'
          ''' % (','.join(['%s=%s' % (f, fieldDict[f]) for f in fieldDict]),
                 ext)
    
    cursor.execute(sql)
    
    cursor.close()

def getExtData(ext):
    record = DEFAULT_EXT.copy()
    record['ext'] = ext

    (extKey, classKey) = getRelatedKeys(ext)

    if classKey == None:
        classKey = extKey

    if classKey.values.has_key(''):
        record['name'] = classKey.values['']
    elif extKey.values.has_key(''):
        record['name'] = extKey.values['']

    if classKey.values.has_key('PerceivedType'):
        record['type'] = classKey.values['PerceivedType']
    elif classKey.values.has_key('Perceived Type'):
        record['type'] = classKey.values['Perceived Type']
    elif extKey.values.has_key('PerceivedType'):
        record['type'] = extKey.values['PerceivedType']
    elif extKey.values.has_key('Perceived Type'):
        record['type'] = extKey.values['Perceived Type']

    if classKey.values.has_key('ContentType'):
        record['mimetype'] = classKey.values['ContentType']
    elif classKey.values.has_key('Content Type'):
        record['mimetype'] = classKey.values['Content Type']
    elif extKey.values.has_key('ContentType'):
        record['mimetype'] = extKey.values['ContentType']
    elif extKey.values.has_key('Content Type'):
        record['mimetype'] = extKey.values['Content Type']

    try:
        record['persistenthandler'] = extKey.getSubkey('PersistentHandler').values['']
    except:
        try:
            record['persistenthandler'] = classKey.getSubkey('PersistentHandler').values['']
        except:
            pass


    try:
        record['clsid'] = classKey.getSubkey('CLSID').values['']
    except:
        try:
            record['clsid'] = extKey.getSubkey('CLSID').values['']
        except:
            pass
    if record['clsid'] == '':
        record['clsid'] = None

    try:
        record['icon'] = classKey.getSubkey('DefaultIcon').values['']
    except:
        try:
            record['icon'] = extKey.getSubkey('DefaultIcon').values['']
        except:
            pass
        

    try:
        record['defaultAction'] = classKey.getSubkey('shell').values['']
    except:
        try:
            record['defaultAction'] = extKey.getSubkey('shell').values['']
        except:
            pass
        
    try:
        record['openCommand'] = classKey.getSubkey('shell').getSubkey('open').getSubkey('command').values['']
    except:
        try:
            record['openCommand'] = extKey.getSubkey('shell').getSubkey('open').getSubkey('command').values['']
        except:
            pass
        
    try:
        record['actionName'] = classKey.getSubkey('shell').getSubkey('open').values['']
    except:
        try:
            record['actionName'] = extKey.getSubkey('shell').getSubkey('open').values['']
        except:
            pass
        
    try:
        record['editCommand'] = classKey.getSubkey('shell').getSubkey('edit').getSubkey('command').values['']
    except:
        try:
            record['editCommand'] = extKey.getSubkey('shell').getSubkey('edit').getSubkey('command').values['']
        except:
            pass

    record['fixed'] = '0'
    try:
        if classKey.name.startswith('_max.'):
            record['fixed'] = '1'
    except:
        pass
    
    return record

def main():
    print "done"
    exts = getExts()

    for ext in exts:    
        record = getExtData(ext)
        insertExtRecord(record)
    conn.commit()
    print "done"
